# frozen_string_literal: true
# Generated by "rake schema:update".  DO NOT EDIT!
# source: https://raw.githubusercontent.com/sendgrid/sendgrid-oai/master/oai.json

module KaniLaser
  SCHEMA = {"type"=>"object",
 "properties"=>
  {"personalizations"=>
    {"type"=>"array",
     "description"=>
      "An array of messages and their metadata. Each object within personalizations can be thought of as an envelope - it defines who should receive an individual message and how that message should be handled.",
     "uniqueItems"=>false,
     "maxItems"=>1000,
     "items"=>
      {"type"=>"object",
       "properties"=>
        {"to"=>
          {"type"=>"array",
           "description"=>
            "An array of recipients. Each object within this array may contain the name, but must always contain the email, of a recipient.",
           "minItems"=>1,
           "maxItems"=>1000,
           "items"=>{"$ref"=>"#/definitions/email_object"}},
         "cc"=>
          {"type"=>"array",
           "description"=>
            "An array of recipients who will receive a copy of your email. Each object within this array may contain the name, but must always contain the email, of a recipient.",
           "maxItems"=>1000,
           "items"=>{"$ref"=>"#/definitions/email_object"}},
         "bcc"=>
          {"type"=>"array",
           "description"=>
            "An array of recipients who will receive a blind carbon copy of your email. Each object within this array may contain the name, but must always contain the email, of a recipient.",
           "maxItems"=>1000,
           "items"=>{"$ref"=>"#/definitions/email_object"}},
         "subject"=>
          {"type"=>"string",
           "description"=>
            "The subject of your email. Char length requirements, according to the RFC - http://stackoverflow.com/questions/1592291/what-is-the-email-subject-length-limit#answer-1592310",
           "minLength"=>1},
         "headers"=>
          {"type"=>"object",
           "description"=>
            "A collection of JSON key/value pairs allowing you to specify specific handling instructions for your email. You may not overwrite the following headers: x-sg-id, x-sg-eid, received, dkim-signature, Content-Type, Content-Transfer-Encoding, To, From, Subject, Reply-To, CC, BCC"},
         "substitutions"=>
          {"type"=>"object",
           "description"=>
            "A collection of key/value pairs following the pattern \"substitution_tag\":\"value to substitute\". All are assumed to be strings. These substitutions will apply to the text and html content of the body of your email, in addition to the `subject` and `reply-to` parameters.",
           "maxProperties"=>10000},
         "custom_args"=>
          {"type"=>"object",
           "description"=>
            "Values that are specific to this personalization that will be carried along with the email and its activity data. Substitutions will not be made on custom arguments, so any string that is entered into this parameter will be assumed to be the custom argument that you would like to be used. May not exceed 10,000 bytes."},
         "send_at"=>
          {"type"=>"integer",
           "description"=>
            "A unix timestamp allowing you to specify when you want your email to be delivered. Scheduling more than 72 hours in advance is forbidden."}},
       "required"=>["to"]},
     "minItems"=>1},
   "from"=>{"$ref"=>"#/definitions/email_object"},
   "reply_to"=>{"$ref"=>"#/definitions/email_object"},
   "subject"=>
    {"type"=>"string",
     "description"=>
      "The global, or “message level”, subject of your email. This may be overridden by personalizations[x].subject.",
     "minLength"=>1},
   "content"=>
    {"type"=>"array",
     "description"=>
      "An array in which you may specify the content of your email. You can include multiple mime types of content, but you must specify at least one mime type. To include more than one mime type, simply add another object to the array containing the `type` and `value` parameters.",
     "items"=>
      {"type"=>"object",
       "properties"=>
        {"type"=>
          {"type"=>"string",
           "description"=>
            "The mime type of the content you are including in your email. For example, “text/plain” or “text/html”.",
           "minLength"=>1},
         "value"=>
          {"type"=>"string",
           "description"=>
            "The actual content of the specified mime type that you are including in your email.",
           "minLength"=>1}},
       "required"=>["type", "value"]}},
   "attachments"=>
    {"type"=>"array",
     "description"=>
      "An array of objects in which you can specify any attachments you want to include.",
     "items"=>
      {"type"=>"object",
       "properties"=>
        {"content"=>
          {"type"=>"string",
           "description"=>"The Base64 encoded content of the attachment.",
           "minLength"=>1},
         "type"=>
          {"type"=>"string",
           "description"=>
            "The mime type of the content you are attaching. For example, “text/plain” or “text/html”.",
           "minLength"=>1},
         "filename"=>
          {"type"=>"string", "description"=>"The filename of the attachment."},
         "disposition"=>
          {"type"=>"string",
           "default"=>"attachment",
           "description"=>
            "The content-disposition of the attachment specifying how you would like the attachment to be displayed. For example, “inline” results in the attached file being displayed automatically within the message while “attachment” results in the attached file requiring some action to be taken before it is displayed (e.g. opening or downloading the file).",
           "enum"=>["inline", "attachment"]},
         "content_id"=>
          {"type"=>"string",
           "description"=>
            "The content id for the attachment. This is used when the disposition is set to “inline” and the attachment is an image, allowing the file to be displayed within the body of your email."}},
       "required"=>["content", "filename"]}},
   "template_id"=>
    {"type"=>"string",
     "description"=>
      "The id of a template that you would like to use. If you use a template that contains a subject and content (either text or html), you do not need to specify those at the personalizations nor message level. "},
   "sections"=>
    {"type"=>"object",
     "description"=>
      "An object of key/value pairs that define block sections of code to be used as substitutions."},
   "headers"=>
    {"type"=>"object",
     "description"=>
      "An object containing key/value pairs of header names and the value to substitute for them. You must ensure these are properly encoded if they contain unicode characters. Must not be one of the reserved headers."},
   "categories"=>
    {"type"=>"array",
     "description"=>
      "An array of category names for this message. Each category name may not exceed 255 characters. ",
     "uniqueItems"=>true,
     "maxItems"=>10,
     "items"=>{"type"=>"string", "maxLength"=>255}},
   "custom_args"=>
    {"type"=>"object",
     "description"=>
      "Values that are specific to the entire send that will be carried along with the email and its activity data. Substitutions will not be made on custom arguments, so any string that is entered into this parameter will be assumed to be the custom argument that you would like to be used. This parameter is overridden by personalizations[x].custom_args if that parameter has been defined. Total custom args size may not exceed 10,000 bytes."},
   "send_at"=>
    {"type"=>"integer",
     "description"=>
      "A unix timestamp allowing you to specify when you want your email to be delivered. This may be overridden by the personalizations[x].send_at parameter. Scheduling more ta 72 hours in advance is forbidden."},
   "batch_id"=>
    {"type"=>"string",
     "description"=>
      "This ID represents a batch of emails to be sent at the same time. Including a batch_id in your request allows you include this email in that batch, and also enables you to cancel or pause the delivery of that batch. For more information, see https://sendgrid.com/docs/API_Reference/Web_API_v3/cancel_schedule_send.html "},
   "asm"=>
    {"type"=>"object",
     "description"=>
      "An object allowing you to specify how to handle unsubscribes.",
     "properties"=>
      {"group_id"=>
        {"type"=>"integer",
         "description"=>"The unsubscribe group to associate with this email."},
       "groups_to_display"=>
        {"type"=>"array",
         "description"=>
          "An array containing the unsubscribe groups that you would like to be displayed on the unsubscribe preferences page.",
         "maxItems"=>25,
         "items"=>{"type"=>"integer"}}},
     "required"=>["group_id"]},
   "ip_pool_name"=>
    {"type"=>"string",
     "description"=>"The IP Pool that you would like to send this email from.",
     "minLength"=>2,
     "maxLength"=>64},
   "mail_settings"=>
    {"type"=>"object",
     "description"=>
      "A collection of different mail settings that you can use to specify how you would like this email to be handled.",
     "properties"=>
      {"bcc"=>
        {"type"=>"object",
         "description"=>
          "This allows you to have a blind carbon copy automatically sent to the specified email address for every email that is sent.",
         "properties"=>
          {"enable"=>
            {"type"=>"boolean",
             "description"=>"Indicates if this setting is enabled."},
           "email"=>
            {"type"=>"string",
             "description"=>
              "The email address that you would like to receive the BCC.",
             "format"=>"email"}}},
       "bypass_list_management"=>
        {"type"=>"object",
         "description"=>
          "Allows you to bypass all unsubscribe groups and suppressions to ensure that the email is delivered to every single recipient. This should only be used in emergencies when it is absolutely necessary that every recipient receives your email.",
         "properties"=>
          {"enable"=>
            {"type"=>"boolean",
             "description"=>"Indicates if this setting is enabled."}}},
       "footer"=>
        {"type"=>"object",
         "description"=>
          "The default footer that you would like included on every email.",
         "properties"=>
          {"enable"=>
            {"type"=>"boolean",
             "description"=>"Indicates if this setting is enabled."},
           "text"=>
            {"type"=>"string",
             "description"=>"The plain text content of your footer."},
           "html"=>
            {"type"=>"string",
             "description"=>"The HTML content of your footer."}}},
       "sandbox_mode"=>
        {"type"=>"object",
         "description"=>
          "This allows you to send a test email to ensure that your request body is valid and formatted correctly.",
         "properties"=>
          {"enable"=>
            {"type"=>"boolean",
             "description"=>"Indicates if this setting is enabled."}}},
       "spam_check"=>
        {"type"=>"object",
         "description"=>
          "This allows you to test the content of your email for spam.",
         "properties"=>
          {"enable"=>
            {"type"=>"boolean",
             "description"=>"Indicates if this setting is enabled."},
           "threshold"=>
            {"type"=>"integer",
             "description"=>
              "The threshold used to determine if your content qualifies as spam on a scale from 1 to 10, with 10 being most strict, or most likely to be considered as spam.",
             "minimum"=>1,
             "maximum"=>10},
           "post_to_url"=>
            {"type"=>"string",
             "description"=>
              "An Inbound Parse URL that you would like a copy of your email along with the spam report to be sent to."}}}}},
   "tracking_settings"=>
    {"type"=>"object",
     "description"=>
      "Settings to determine how you would like to track the metrics of how your recipients interact with your email.",
     "properties"=>
      {"click_tracking"=>
        {"type"=>"object",
         "description"=>
          "Allows you to track whether a recipient clicked a link in your email.",
         "properties"=>
          {"enable"=>
            {"type"=>"boolean",
             "description"=>"Indicates if this setting is enabled."},
           "enable_text"=>
            {"type"=>"boolean",
             "description"=>
              "Indicates if this setting should be included in the text/plain portion of your email."}}},
       "open_tracking"=>
        {"type"=>"object",
         "description"=>
          "Allows you to track whether the email was opened or not, but including a single pixel image in the body of the content. When the pixel is loaded, we can log that the email was opened.",
         "properties"=>
          {"enable"=>
            {"type"=>"boolean",
             "description"=>"Indicates if this setting is enabled."},
           "substitution_tag"=>
            {"type"=>"string",
             "description"=>
              "Allows you to specify a substitution tag that you can insert in the body of your email at a location that you desire. This tag will be replaced by the open tracking pixel."}}},
       "subscription_tracking"=>
        {"type"=>"object",
         "description"=>
          "Allows you to insert a subscription management link at the bottom of the text and html bodies of your email. If you would like to specify the location of the link within your email, you may use the substitution_tag.",
         "properties"=>
          {"enable"=>
            {"type"=>"boolean",
             "description"=>"Indicates if this setting is enabled."},
           "text"=>
            {"type"=>"string",
             "description"=>
              "Text to be appended to the email, with the subscription tracking link. You may control where the link is by using the tag <% %>"},
           "html"=>
            {"type"=>"string",
             "description"=>
              "HTML to be appended to the email, with the subscription tracking link. You may control where the link is by using the tag <% %>"},
           "substitution_tag"=>
            {"type"=>"string",
             "description"=>
              "A tag that will be replaced with the unsubscribe URL. for example: [unsubscribe_url]. If this parameter is used, it will override both the `text` and `html` parameters. The URL of the link will be placed at the substitution tag’s location, with no additional formatting."}}},
       "ganalytics"=>
        {"type"=>"object",
         "description"=>
          "Allows you to enable tracking provided by Google Analytics.",
         "properties"=>
          {"enable"=>
            {"type"=>"boolean",
             "description"=>"Indicates if this setting is enabled."},
           "utm_source"=>
            {"type"=>"string",
             "description"=>
              "Name of the referrer source. (e.g. Google, SomeDomain.com, or Marketing Email)"},
           "utm_medium"=>
            {"type"=>"string",
             "description"=>"Name of the marketing medium. (e.g. Email)"},
           "utm_term"=>
            {"type"=>"string",
             "description"=>"Used to identify any paid keywords.\t"},
           "utm_content"=>
            {"type"=>"string",
             "description"=>
              "Used to differentiate your campaign from advertisements.\t"},
           "utm_campaign"=>
            {"type"=>"string",
             "description"=>"The name of the campaign.\t"}}}}}},
 "required"=>["personalizations", "from"],
 "example"=>
  {"personalizations"=>
    [{"to"=>[{"email"=>"john.doe@example.com", "name"=>"John Doe"}],
      "subject"=>"Hello, World!"}],
   "from"=>{"email"=>"sam.smith@example.com", "name"=>"Sam Smith"},
   "reply_to"=>{"email"=>"sam.smith@example.com", "name"=>"Sam Smith"},
   "subject"=>"Hello, World!",
   "content"=>
    [{"type"=>"text/html", "value"=>"<html><p>Hello, world!</p></html>"}]},
 "definitions"=>
  {"mail_settings_spam_check"=>
    {"title"=>"Mail Settings: Spam Check",
     "type"=>"object",
     "properties"=>
      {"enabled"=>
        {"type"=>"boolean",
         "description"=>
          "Indicates if your Spam Checker mail setting is enabled."},
       "max_score"=>
        {"type"=>"integer",
         "default"=>5,
         "description"=>
          "The spam threshold. Can range from 1 to 10. The lower the number, the more strict the filtering.",
         "minimum"=>1,
         "maximum"=>10},
       "url"=>
        {"type"=>"string",
         "description"=>
          "The inbound parse URL where you would like the spam messages to be sent to."}},
     "required"=>["enabled"],
     "example"=>
      {"enabled"=>false, "max_score"=>6, "url"=>"http://example.com"}},
   "suppression_group_unsubscribes"=>
    {"title"=>"Suppressions: Suppression Group with Unsubscribes",
     "allOf"=>
      [{"$ref"=>"#/definitions/suppression_group"},
       {"properties"=>
         {"unsubscribes"=>
           {"type"=>"integer",
            "description"=>"The unsubscribes associated with this group."}},
        "required"=>["unsubscribes"]}],
     "type"=>"object"},
   "partner_settings_new_relic"=>
    {"title"=>"Partner Settings: New Relic",
     "type"=>"object",
     "properties"=>
      {"enable_subuser_statistics"=>
        {"type"=>"boolean",
         "description"=>
          "Indicates if your subuser statistics will be sent to your New Relic Dashboard."},
       "enabled"=>
        {"type"=>"boolean",
         "description"=>"Indicates if this setting is enabled. "},
       "license_key"=>
        {"type"=>"string",
         "description"=>
          "The license key provided with your New Relic account."}},
     "required"=>["enabled", "license_key"]},
   "subscription_tracking_settings"=>
    {"title"=>"Settings: Subscription Tracking",
     "type"=>"object",
     "properties"=>
      {"enabled"=>
        {"type"=>"boolean",
         "description"=>"Indicates if subscription tracking is enabled."},
       "html_content"=>
        {"type"=>"string",
         "description"=>
          "The information and HTML for your unsubscribe link. "},
       "landing"=>
        {"type"=>"string",
         "description"=>
          "The HTML that will be displayed on the page that your customers will see after clicking unsubscribe, hosted on SendGrid’s server."},
       "plain_content"=>
        {"type"=>"string",
         "description"=>
          "The information in plain text for your unsubscribe link. You should have the “<% %>” tag in your content, otherwise the user will have no URL for unsubscribing."},
       "replace"=>
        {"type"=>"string",
         "description"=>
          "Your custom defined replacement tag for your templates. Use this tag to place your unsubscribe content anywhere in your emailtemplate."},
       "url"=>
        {"type"=>"string",
         "description"=>
          "The URL where you would like your users sent to unsubscribe."}}},
   "campaign_response"=>
    {"title"=>"Campaigns Response",
     "allOf"=>
      [{"$ref"=>"#/definitions/campaign_request"},
       {"type"=>"object",
        "properties"=>
         {"status"=>
           {"type"=>"string", "description"=>"The status of your campaign."},
          "id"=>{"type"=>"integer"}},
        "required"=>["status"]}]},
   "contactdb_recipient_response"=>
    {"title"=>"ContactDB: Recipient response",
     "type"=>"object",
     "properties"=>
      {"error_count"=>
        {"type"=>"number",
         "default"=>0,
         "description"=>"The number of errors found while adding recipients."},
       "error_indices"=>
        {"type"=>"array",
         "default"=>[],
         "description"=>
          "The indices of the recipient(s) sent that caused the error. ",
         "items"=>{"type"=>"number"}},
       "new_count"=>
        {"type"=>"number",
         "default"=>0,
         "description"=>"The count of new recipients added to the contactdb."},
       "persisted_recipients"=>
        {"type"=>"array",
         "default"=>[],
         "description"=>
          "The recipient IDs of the recipients that already existed from this request.",
         "items"=>{"type"=>"string"}},
       "updated_count"=>
        {"type"=>"number",
         "default"=>0,
         "description"=>"The recipients who were updated from this request."},
       "errors"=>
        {"type"=>"array",
         "items"=>
          {"type"=>"object",
           "properties"=>
            {"message"=>{"type"=>"string"},
             "error_indices"=>
              {"type"=>"array", "items"=>{"type"=>"number"}}}}}},
     "required"=>
      ["error_count", "new_count", "persisted_recipients", "updated_count"],
     "example"=>
      {"error_count"=>1,
       "error_indices"=>[2],
       "new_count"=>2,
       "persisted_recipients"=>["YUBh", "bWlsbGVyQG1pbGxlci50ZXN0"],
       "updated_count"=>0,
       "errors"=>[{"message"=>"Invalid email.", "error_indices"=>[2]}]}},
   "stats"=>
    {"title"=>"Stats",
     "type"=>"array",
     "items"=>
      {"type"=>"object",
       "properties"=>
        {"date"=>
          {"type"=>"string",
           "description"=>"The date that the statistics were gathered."},
         "stats"=>
          {"type"=>"array",
           "description"=>"The list of statistics.",
           "items"=>
            {"type"=>"object",
             "properties"=>
              {"type"=>
                {"type"=>"string", "description"=>"The type of segmentation."},
               "name"=>
                {"type"=>"string",
                 "description"=>"The name of the specific segmentation."},
               "metrics"=>
                {"type"=>"object",
                 "description"=>"The individual events and their statistics.",
                 "properties"=>
                  {"blocks"=>
                    {"type"=>"integer",
                     "description"=>
                      "The number of emails that were not allowed to be delivered by ISPs."},
                   "bounce_drops"=>
                    {"type"=>"integer",
                     "description"=>
                      "The number of emails that were dropped because of a bounce."},
                   "bounces"=>
                    {"type"=>"integer",
                     "description"=>
                      "The number of emails that bounced instead of being delivered."},
                   "clicks"=>
                    {"type"=>"integer",
                     "description"=>
                      "The number of links that were clicked in your emails."},
                   "deferred"=>
                    {"type"=>"integer",
                     "description"=>
                      "The number of emails that temporarily could not be delivered. "},
                   "delivered"=>
                    {"type"=>"integer",
                     "description"=>
                      "The number of emails SendGrid was able to confirm were actually delivered to a recipient."},
                   "invalid_emails"=>
                    {"type"=>"integer",
                     "description"=>
                      "The number of recipients who had malformed email addresses or whose mail provider reported the address as invalid."},
                   "opens"=>
                    {"type"=>"integer",
                     "description"=>
                      "The total number of times your emails were opened by recipients."},
                   "processed"=>
                    {"type"=>"integer",
                     "description"=>
                      "Requests from your website, application, or mail client via SMTP Relay or the API that SendGrid processed."},
                   "requests"=>
                    {"type"=>"integer",
                     "description"=>
                      "The number of emails that were requested to be delivered."},
                   "spam_report_drops"=>
                    {"type"=>"integer",
                     "description"=>
                      "The number of emails that were dropped due to a recipient previously marking your emails as spam."},
                   "spam_reports"=>
                    {"type"=>"integer",
                     "description"=>
                      "The number of recipients who marked your email as spam."},
                   "unique_clicks"=>
                    {"type"=>"integer",
                     "description"=>
                      "The number of unique recipients who clicked links in your emails."},
                   "unique_opens"=>
                    {"type"=>"integer",
                     "description"=>
                      "The number of unique recipients who opened your emails."},
                   "unsubscribe_drops"=>
                    {"type"=>"integer",
                     "description"=>
                      "The number of emails dropped due to a recipient unsubscribing from your emails."},
                   "unsubscribes"=>
                    {"type"=>"integer",
                     "description"=>
                      "The number of recipients who unsubscribed from your emails."}}}}}}}}},
   "contactdb_segments_conditions"=>
    {"title"=>"ContactDB: Segments: Conditions",
     "type"=>"object",
     "properties"=>
      {"field"=>{"type"=>"string"},
       "value"=>{"type"=>"string"},
       "operator"=>
        {"type"=>"string", "enum"=>["eq", "ne", "lt", "gt", "contains"]},
       "and_or"=>{"type"=>"string", "enum"=>["and", "or", ""]}},
     "required"=>["field", "value", "operator"]},
   "suppression_bounce"=>
    {"title"=>"Suppression: Bounce",
     "type"=>"object",
     "properties"=>
      {"created"=>
        {"type"=>"number",
         "description"=>
          "The unix timestamp for when the bounce record was created at SendGrid."},
       "email"=>{"type"=>"string"},
       "reason"=>
        {"type"=>"string",
         "description"=>
          "The reason for the bounce. This typically will be a bounce code, an enhanced code, and a description."},
       "status"=>
        {"type"=>"string", "description"=>"Enhanced SMTP bounce response"}},
     "example"=>
      {"created"=>1250337600,
       "email"=>"example@example.com",
       "reason"=>
        "550 5.1.1 The email account that you tried to reach does not exist. Please try double-checking the recipient's email address for typos or unnecessary spaces. Learn more at  https://support.google.com/mail/answer/6596 o186si2389584ioe.63 - gsmtp ",
       "status"=>"5.1.1"}},
   "ip_whitelabel"=>
    {"title"=>"Whitelabel - IPs",
     "type"=>"object",
     "properties"=>
      {"id"=>
        {"type"=>"integer", "description"=>"The id of the IP whitelabel."},
       "ip"=>
        {"type"=>"string",
         "description"=>
          "The IP address that this whitelabel was created for."},
       "rdns"=>
        {"type"=>"string",
         "description"=>
          "The reverse DNS record for the IP address. This points to the IP whitelabel subdomain."},
       "users"=>
        {"type"=>"array",
         "description"=>"The users who are able to send mail from the IP.",
         "items"=>
          {"type"=>"object",
           "properties"=>
            {"username"=>
              {"type"=>"string",
               "description"=>
                "The username of the user who can send mail from this IP."},
             "user_id"=>
              {"type"=>"integer",
               "description"=>
                "The ID of the user who can send mail from this IP."}},
           "required"=>["username", "user_id"]}},
       "subdomain"=>
        {"type"=>"string",
         "description"=>
          "The subdomain created for this IP whitelabel. This is where the rDNS record points."},
       "domain"=>
        {"type"=>"string", "description"=>"The root, or sending, domain."},
       "valid"=>
        {"type"=>"boolean",
         "description"=>"Indicates if this is a valid whitelabel."},
       "legacy"=>
        {"type"=>"boolean",
         "description"=>
          "Indicates if this whitelabel was created using the legacy whitelabel tool."},
       "a_record"=>
        {"type"=>"object",
         "required"=>["valid", "type", "host", "data"],
         "properties"=>
          {"valid"=>
            {"type"=>"boolean",
             "description"=>"Indicates if the a_record is valid."},
           "type"=>
            {"type"=>"string", "description"=>"The type of DNS record."},
           "host"=>
            {"type"=>"string",
             "description"=>
              "This is the web address that will be mapped to the IP address."},
           "data"=>
            {"type"=>"string",
             "description"=>"The IP address being whitelabeled."}}}},
     "required"=>
      ["id",
       "ip",
       "rdns",
       "users",
       "subdomain",
       "domain",
       "valid",
       "legacy",
       "a_record"],
     "example"=>
      {"id"=>1,
       "ip"=>"192.168.1.1",
       "rdns"=>"o1.email.example.com",
       "users"=>
        [{"username"=>"john@example.com", "user_id"=>7},
         {"username"=>"jane@example.com", "user_id"=>8}],
       "subdomain"=>"email",
       "domain"=>"example.com",
       "valid"=>true,
       "legacy"=>false,
       "a_record"=>
        {"valid"=>true,
         "type"=>"a",
         "host"=>"o1.email.example.com",
         "data"=>"192.168.1.1"}}},
   "contacts"=>
    {"title"=>"Contacts",
     "type"=>"object",
     "properties"=>
      {"address"=>{"type"=>"string"},
       "address2"=>{"type"=>"object"},
       "city"=>{"type"=>"string"},
       "company"=>{"type"=>"string"},
       "country"=>{"type"=>"string"},
       "email"=>{"type"=>"string"},
       "first_name"=>{"type"=>"string"},
       "last_name"=>{"type"=>"string"},
       "phone"=>{"type"=>"string"},
       "state"=>{"type"=>"string"},
       "zip"=>{"type"=>"string"}}},
   "senderID"=>
    {"title"=>"Sender ID",
     "type"=>"object",
     "properties"=>
      {"id"=>
        {"type"=>"integer",
         "description"=>"The unique identifier of the sender identity."},
       "nickname"=>
        {"type"=>"string",
         "description"=>
          "A nickname for the sender identity. Not used for sending."},
       "from"=>
        {"type"=>"object",
         "properties"=>
          {"email"=>
            {"type"=>"string",
             "description"=>
              "This is where the email will appear to originate from for your recipient"},
           "name"=>
            {"type"=>"string",
             "description"=>
              "This is the name appended to the from email field. IE - Your name or company name."}},
         "required"=>["email"]},
       "reply_to"=>
        {"type"=>"object",
         "properties"=>
          {"email"=>
            {"type"=>"string",
             "description"=>
              "This is the email that your recipient will reply to."},
           "name"=>
            {"type"=>"string",
             "description"=>
              "This is the name appended to the reply to email field. IE - Your name or company name."}}},
       "address"=>
        {"type"=>"string",
         "description"=>"The physical address of the sender identity."},
       "address_2"=>
        {"type"=>"string",
         "description"=>"Additional sender identity address information."},
       "city"=>
        {"type"=>"string", "description"=>"The city of the sender identity."},
       "state"=>
        {"type"=>"string", "description"=>"The state of the sender identity."},
       "zip"=>
        {"type"=>"string",
         "description"=>"The zipcode of the sender identity."},
       "country"=>
        {"type"=>"string",
         "description"=>"The country of the sender identity."},
       "verified"=>
        {"type"=>"boolean",
         "description"=>
          "If the sender identity is verified or not. Only verified sender identities can be used to send email."},
       "updated_at"=>
        {"type"=>"integer",
         "description"=>"The time the sender identity was last updated."},
       "created_at"=>
        {"type"=>"integer",
         "description"=>"The time the sender identity was created."},
       "locked"=>
        {"type"=>"boolean",
         "description"=>
          "A sender identity is locked when it is associated to a campaign in the Draft, Scheduled, or In Progress status. You cannot update or delete a locked sender identity."}},
     "required"=>["nickname", "address", "city", "country"],
     "example"=>
      {"id"=>1,
       "nickname"=>"My Sender ID",
       "from"=>{"email"=>"from@example.com", "name"=>"Example INC"},
       "reply_to"=>{"email"=>"replyto@example.com", "name"=>"Example INC"},
       "address"=>"123 Elm St.",
       "address_2"=>"Apt. 456",
       "city"=>"Denver",
       "state"=>"Colorado",
       "zip"=>"80202",
       "country"=>"United States",
       "verified"=>true,
       "updated_at"=>1449872165,
       "created_at"=>1449872165,
       "locked"=>false}},
   "global:empty_request"=>
    {"title"=>"Global: Request Empty Body", "type"=>"null"},
   "contactdb_custom_field"=>
    {"title"=>"ContactDB Custom field schema.",
     "type"=>"object",
     "properties"=>
      {"name"=>{"type"=>"string", "description"=>"The name of the field"},
       "type"=>
        {"type"=>"string",
         "description"=>"The type of the field.",
         "enum"=>["date", "text", "number"]}},
     "example"=>{"name"=>"first_name", "type"=>"text"}},
   "whitelabel:domain_spf"=>
    {"title"=>"Whitelabel - Domain",
     "type"=>"object",
     "properties"=>
      {"id"=>
        {"type"=>"integer", "description"=>"The ID of the domain whitelabel."},
       "domain"=>
        {"type"=>"string",
         "description"=>"The domain that this whitelabel was created for."},
       "subdomain"=>
        {"type"=>"string",
         "description"=>
          "The subdomain that was used to create this whitelabel."},
       "username"=>
        {"type"=>"string",
         "description"=>
          "The username of the account that this whitelabel is associated with."},
       "user_id"=>
        {"type"=>"integer",
         "description"=>
          "The user_id of the account that this whitelabel is associated with."},
       "ips"=>
        {"type"=>"array",
         "description"=>
          "The IP addresses that are included in the SPF record for this whitelabel.",
         "items"=>{}},
       "custom_spf"=>
        {"type"=>"boolean",
         "description"=>"Indicates if this whitelabel uses custom SPF."},
       "default"=>
        {"type"=>"boolean",
         "description"=>"Indicates if this is the default whitelabel."},
       "legacy"=>
        {"type"=>"boolean",
         "description"=>
          "Indicates if this whitelabel was created using the legacy whitelabel tool."},
       "automatic_security"=>
        {"type"=>"boolean",
         "description"=>
          "Indicates if this whitelabel uses automated security."},
       "valid"=>
        {"type"=>"boolean",
         "description"=>"Indicates if this is a valid whitelabel."},
       "dns"=>
        {"type"=>"object",
         "description"=>"The DNS records for this whitelabel.",
         "required"=>["mail_server", "subdomain_spf", "domain_spf", "dkim"],
         "properties"=>
          {"mail_server"=>
            {"type"=>"object",
             "description"=>
              "Designates which mail server is responsible for accepting messages from a domain.",
             "required"=>["host", "type", "data", "valid"],
             "properties"=>
              {"host"=>
                {"type"=>"string",
                 "description"=>"The domain sending the messages."},
               "type"=>
                {"type"=>"string", "description"=>"They type of DNS record."},
               "data"=>
                {"type"=>"string",
                 "description"=>
                  "The mail server responsible for accepting messages from the sending domain."},
               "valid"=>
                {"type"=>"boolean",
                 "description"=>"Indicates if this is a valid DNS record."}}},
           "subdomain_spf"=>
            {"type"=>"object",
             "description"=>
              "The SPF record for the subdomain used to create this whitelabel.",
             "required"=>["host", "type", "data", "valid"],
             "properties"=>
              {"host"=>
                {"type"=>"string",
                 "description"=>
                  "The domain that this SPF record will be used to authenticate."},
               "type"=>
                {"type"=>"string",
                 "description"=>"The type of data in the SPF record."},
               "data"=>{"type"=>"string", "description"=>"The SPF record."},
               "valid"=>
                {"type"=>"boolean",
                 "description"=>"Indicates if this is a valid SPF record."}}},
           "domain_spf"=>
            {"type"=>"object",
             "description"=>"The SPF record for the root domain.",
             "required"=>["host", "type", "data", "valid"],
             "properties"=>
              {"host"=>
                {"type"=>"string",
                 "description"=>
                  "The root domain that this SPF record will be used to authenticate."},
               "type"=>
                {"type"=>"string",
                 "description"=>"The type of data in the SPF record."},
               "data"=>{"type"=>"string", "description"=>"The SPF record."},
               "valid"=>
                {"type"=>"boolean",
                 "description"=>"Indicates if the SPF record is valid."}}},
           "dkim"=>
            {"type"=>"object",
             "description"=>
              "The DKIM record for messages sent using this whitelabel.",
             "required"=>["host", "type", "data", "valid"],
             "properties"=>
              {"host"=>
                {"type"=>"string",
                 "description"=>"The DNS labels for the DKIM signature."},
               "type"=>
                {"type"=>"string",
                 "description"=>"The type of data in the DKIM record."},
               "data"=>{"type"=>"string", "description"=>"The DKIM record."},
               "valid"=>
                {"type"=>"boolean",
                 "description"=>"Indicates if the DKIM record is valid."}}}}}},
     "required"=>
      ["id",
       "domain",
       "subdomain",
       "username",
       "user_id",
       "ips",
       "custom_spf",
       "default",
       "legacy",
       "automatic_security",
       "valid",
       "dns"]},
   "subuser"=>
    {"title"=>"List all Subusers for a parent response",
     "type"=>"object",
     "properties"=>
      {"disabled"=>
        {"type"=>"boolean",
         "description"=>"Whether or not the user is enabled or disabled."},
       "id"=>{"type"=>"number", "description"=>"The ID of this subuser."},
       "username"=>
        {"type"=>"string",
         "description"=>"The name by which this subuser will be referred."},
       "email"=>
        {"type"=>"string",
         "description"=>"The email address to contact this subuser.",
         "format"=>"email"}},
     "required"=>["disabled", "id", "username", "email"],
     "example"=>
      {"disabled"=>false,
       "email"=>"example@example.com",
       "id"=>1234,
       "username"=>"example_subuser"}},
   "mail_settings_address_whitelabel"=>
    {"title"=>"Mail Settings: Address Whitelabel",
     "type"=>"object",
     "properties"=>
      {"enabled"=>
        {"type"=>"boolean",
         "description"=>
          "Indicates if you have an email address whitelist enabled. "},
       "list"=>
        {"type"=>"array",
         "description"=>
          "All email address that are currently on the whitelist.",
         "items"=>{"type"=>"string"}}},
     "example"=>
      {"enabled"=>true, "list"=>["email1@example.com", "example.com"]}},
   "link_whitelabel"=>
    {"title"=>"Whitelabel - Links",
     "type"=>"object",
     "properties"=>
      {"id"=>
        {"type"=>"integer", "description"=>"The id of the link whitelabel."},
       "domain"=>
        {"type"=>"string",
         "description"=>"The root domain for this link whitelabel."},
       "subdomain"=>
        {"type"=>"string",
         "description"=>
          "The subdomain used to generate the DNS records for this link whitelabel. This subdomain must be different from the subdomain used for your domain whitelabel."},
       "username"=>
        {"type"=>"string",
         "description"=>
          "The username of the account that this link whitelabel is associated with."},
       "user_id"=>
        {"type"=>"integer",
         "description"=>
          "The id of the user that this whitelabel is associated with."},
       "default"=>
        {"type"=>"boolean",
         "description"=>"Indicates if this is the default link whitelabel.",
         "enum"=>[true, false]},
       "valid"=>
        {"type"=>"boolean",
         "description"=>"Indicates if this link whitelabel is valid.",
         "enum"=>[true, false]},
       "legacy"=>
        {"type"=>"boolean",
         "description"=>
          "Indicates if this link whitelabel was created using the legacy whitelabel tool.",
         "enum"=>[true, false]},
       "dns"=>
        {"type"=>"object",
         "description"=>"The DNS records generated for this link whitelabel.",
         "required"=>["domain_cname"],
         "properties"=>
          {"domain_cname"=>
            {"type"=>"object",
             "description"=>
              "The DNS record generated to point to your link whitelabel subdomain.",
             "required"=>["valid", "type", "host", "data"],
             "properties"=>
              {"valid"=>
                {"type"=>"boolean",
                 "description"=>"Indicates if the DNS record is valid.",
                 "enum"=>[true, false]},
               "type"=>
                {"type"=>"string",
                 "description"=>"The type of DNS record that was generate.",
                 "enum"=>["cname", "txt", "mx"]},
               "host"=>
                {"type"=>"string",
                 "description"=>
                  "The domain that this whitelabel will use when whitelabeling the links in your email."},
               "data"=>
                {"type"=>"string",
                 "description"=>"The domain that the DNS record points to."}}},
           "owner_cname"=>
            {"type"=>"object",
             "description"=>
              "The DNS record generated to verify who created the link whitelabel.",
             "properties"=>
              {"valid"=>
                {"type"=>"boolean",
                 "description"=>"Indicates if the DNS record is valid.",
                 "enum"=>[true, false]},
               "type"=>
                {"type"=>"string",
                 "description"=>"The type of DNS record generated.",
                 "enum"=>["cname", "txt", "mx"]},
               "host"=>
                {"type"=>"string",
                 "description"=>
                  "Used to verify the link whitelabel. The subdomain of this domain is the user id of the user who created the link whitelabel."},
               "data"=>
                {"type"=>"string",
                 "description"=>"The domain that the DNS record points to."}},
             "required"=>["valid", "host", "data"]}}}},
     "required"=>
      ["id",
       "domain",
       "subdomain",
       "username",
       "user_id",
       "default",
       "valid",
       "legacy",
       "dns"]},
   "email_object"=>
    {"title"=>"Email Object",
     "type"=>"object",
     "properties"=>
      {"email"=>{"type"=>"string", "format"=>"email"},
       "name"=>
        {"type"=>"string",
         "description"=>
          "The name of the person to whom you are sending an email."}},
     "required"=>["email"]},
   "api_key_name_id_scopes"=>
    {"title"=>"API Key Name, ID, and Scopes",
     "allOf"=>
      [{"type"=>"object",
        "properties"=>
         {"scopes"=>
           {"type"=>"array",
            "description"=>"The permissions this API Key has access to.",
            "items"=>{"type"=>"string"}}}},
       {"$ref"=>"#/definitions/api_key_name_id"}],
     "example"=>
      {"api_key_id"=>"qfTQ6KG0QBiwWdJ0-pCLCA",
       "name"=>"A New Hope",
       "scopes"=>["user.profile.read", "user.profile.update"]}},
   "contactdb_segments"=>
    {"title"=>"Create a Segment request",
     "type"=>"object",
     "properties"=>
      {"name"=>{"type"=>"string", "description"=>"The name of this segment."},
       "list_id"=>
        {"type"=>"integer",
         "description"=>
          "The list id from which to make this segment. Not including this ID will mean your segment is created from the main contactdb rather than a list."},
       "conditions"=>
        {"type"=>"array",
         "description"=>
          "The conditions for a recipient to be included in this segment.",
         "items"=>{"$ref"=>"#/definitions/contactdb_segments_conditions"}},
       "recipient_count"=>
        {"type"=>"number",
         "description"=>
          "The count of recipients in this list. This is not included on creation of segments."}},
     "required"=>["name", "conditions"],
     "example"=>
      {"name"=>"Last Name Miller",
       "list_id"=>4,
       "conditions"=>
        [{"field"=>"last_name",
          "value"=>"Miller",
          "operator"=>"eq",
          "and_or"=>""},
         {"field"=>"last_clicked",
          "value"=>"01/02/2015",
          "operator"=>"gt",
          "and_or"=>"and"},
         {"field"=>"clicks.campaign_identifier",
          "value"=>"513",
          "operator"=>"eq",
          "and_or"=>"or"}],
       "recipient_count"=>1234}},
   "api_key_name_id"=>
    {"title"=>"API Key Name and ID",
     "type"=>"object",
     "properties"=>
      {"api_key_id"=>
        {"type"=>"string", "description"=>"The ID of your API Key. "},
       "name"=>{"type"=>"string", "description"=>"The name of your API Key."}},
     "example"=>
      {"api_key_id"=>"qfTQ6KG0QBiwWdJ0-pCLCA", "name"=>"A New Hope"}},
   "advanced_stats_opens"=>
    {"title"=>"Stats: Advanced Stats with Opens",
     "type"=>"object",
     "properties"=>
      {"date"=>
        {"type"=>"string",
         "description"=>"The date that the events occurred."},
       "stats"=>
        {"type"=>"array",
         "description"=>"The statistics of the email events.",
         "items"=>
          {"type"=>"object",
           "properties"=>
            {"type"=>
              {"type"=>"string", "description"=>"The type of segmentation."},
             "name"=>
              {"type"=>"string",
               "description"=>"The name of the specific segmentation."},
             "metrics"=>
              {"type"=>"object",
               "description"=>"The individual events and their stats.",
               "required"=>["opens", "unique_opens"],
               "properties"=>
                {"opens"=>
                  {"type"=>"integer",
                   "description"=>
                    "The total number of times your emails were opened by recipients."},
                 "unique_opens"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of unique recipients who opened your emails."}}}},
           "required"=>["type", "name", "metrics"]}}},
     "required"=>["date", "stats"]},
   "mail_settings_template"=>
    {"title"=>"Mail Settings: Template",
     "type"=>"object",
     "properties"=>
      {"enabled"=>
        {"type"=>"boolean",
         "description"=>
          "Indicates if the legacy email template setting is enabled."},
       "html_content"=>
        {"type"=>"string",
         "description"=>
          "The HTML content that you want to use for your legacy email template."}},
     "example"=>
      {"enabled"=>false, "html_content"=>"<p><% body %>Example</p>\n"}},
   "ip_warmup_response"=>
    {"title"=>"IP Warmup: IP",
     "type"=>"array",
     "items"=>
      {"type"=>"object",
       "properties"=>
        {"ip"=>{"type"=>"string", "description"=>"The IP address."},
         "start_date"=>
          {"type"=>"integer",
           "description"=>
            "A Unix timestamp indicating when the IP address was entered into warmup mode."}},
       "required"=>["ip", "start_date"]},
     "example"=>[{"ip"=>"0.0.0.0", "start_date"=>1409616000}]},
   "advanced_stats_mailbox_provider"=>
    {"title"=>"Stats: Advanced Stats for Mailbox Provider",
     "type"=>"object",
     "properties"=>
      {"date"=>
        {"type"=>"string",
         "description"=>"The date that the events occurred."},
       "stats"=>
        {"type"=>"array",
         "description"=>"The statistics of the email events.",
         "items"=>
          {"type"=>"object",
           "properties"=>
            {"type"=>
              {"type"=>"string", "description"=>"The type of segmentation."},
             "name"=>
              {"type"=>"string",
               "description"=>"The name of the specific segmentation."},
             "metrics"=>
              {"type"=>"object",
               "description"=>"The individual events and their stats.",
               "required"=>
                ["clicks",
                 "opens",
                 "unique_clicks",
                 "unique_opens",
                 "blocks",
                 "bounces",
                 "deferred",
                 "delivered",
                 "drops",
                 "spam_reports"],
               "properties"=>
                {"clicks"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of links that were clicked in your emails."},
                 "opens"=>
                  {"type"=>"integer",
                   "description"=>
                    "The total number of times your emails were opened by recipients."},
                 "unique_clicks"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of unique recipients who clicked links in your emails."},
                 "unique_opens"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of unique recipients who opened your emails."},
                 "blocks"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of emails that were not allowed to be delivered by ISPs."},
                 "bounces"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of emails that bounced instead of being delivered."},
                 "deferred"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of emails that temporarily could not be delivered."},
                 "delivered"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of emails SendGrid was able to confirm were actually delivered to a recipient."},
                 "drops"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of emails that were not delivered due to the recipient email address being on a suppression list."},
                 "spam_reports"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of recipients who marked your email as spam."}}}},
           "required"=>["type", "name", "metrics"]}}},
     "required"=>["date", "stats"]},
   "global:ErrorResponse"=>
    {"title"=>"Global: Error Response",
     "type"=>"object",
     "properties"=>
      {"errors"=>
        {"type"=>"array",
         "items"=>
          {"type"=>"object",
           "properties"=>
            {"field"=>
              {"type"=>["string", "null"],
               "description"=>"The field that generated the error."},
             "message"=>
              {"type"=>"string", "description"=>"The error message."}},
           "required"=>["message"]}}},
     "example"=>
      {"errors"=>[{"field"=>"field_name", "message"=>"Some message here"}]}},
   "contactdb_custom_field_with_id"=>
    {"title"=>"ContactDB Custom field schema with ID.",
     "allOf"=>
      [{"$ref"=>"#/definitions/contactdb_custom_field"},
       {"type"=>"object",
        "properties"=>
         {"id"=>
           {"type"=>"number",
            "description"=>"The ID of the custom field."}}}]},
   "monitor"=>
    {"title"=>"Create monitor settings request",
     "type"=>"object",
     "properties"=>
      {"email"=>
        {"type"=>"string",
         "description"=>
          "The email address to send emails at the frequency specified for monitoring.",
         "format"=>"email"},
       "frequency"=>
        {"type"=>"number",
         "description"=>
          "The frequency by which to send the emails. An email will be sent, every time your subuser sends this {frequency} emails. "}},
     "required"=>["email", "frequency"],
     "example"=>{"email"=>"example@example.com", "frequency"=>50000}},
   "errors"=>
    {"title"=>"Error Schema",
     "type"=>"object",
     "properties"=>
      {"errors"=>
        {"type"=>"array",
         "items"=>
          {"type"=>"object",
           "properties"=>
            {"field"=>
              {"type"=>["null", "string"],
               "description"=>"The field that has the error."},
             "message"=>
              {"type"=>"string",
               "description"=>"The message the API caller will receive."}}}}}},
   "ip_pool"=>
    {"title"=>"IP Pools: Pool",
     "type"=>"object",
     "properties"=>
      {"name"=>
        {"type"=>"string",
         "description"=>"The name of the IP pool.",
         "maxLength"=>64}},
     "required"=>["name"]},
   "google_analytics_settings"=>
    {"title"=>"Settings: Google Analytics",
     "type"=>"object",
     "properties"=>
      {"enabled"=>
        {"type"=>"boolean",
         "description"=>"Indicates if Google Analytics is enabled."},
       "utm_campaign"=>
        {"type"=>"string", "description"=>"The name of the campaign."},
       "utm_content"=>
        {"type"=>"string", "description"=>"Used to differentiate ads"},
       "utm_medium"=>
        {"type"=>"string",
         "description"=>"Name of the marketing medium (e.g. \"Email\")."},
       "utm_source"=>
        {"type"=>"string", "description"=>"Name of the referrer source. "},
       "utm_term"=>{"type"=>"string", "description"=>"Any paid keywords."}},
     "example"=>
      {"enabled"=>true,
       "utm_source"=>"sendgrid.com",
       "utm_medium"=>"email",
       "utm_term"=>"",
       "utm_content"=>"",
       "utm_campaign"=>"website"}},
   "event_webhook_settings"=>
    {"title"=>"Webhooks: Event Webhook Settings",
     "type"=>"object",
     "properties"=>
      {"enabled"=>
        {"type"=>"boolean",
         "description"=>"Indicates if the event webhook is enabled."},
       "url"=>
        {"type"=>"string",
         "description"=>"The URL that you want the event webhook to POST to."},
       "group_resubscribe"=>
        {"type"=>"boolean",
         "description"=>
          "Recipient resubscribes to specific group by updating preferences. You need to enable Subscription Tracking for getting this type of event."},
       "delivered"=>
        {"type"=>"boolean",
         "description"=>
          "Message has been successfully delivered to the receiving server."},
       "group_unsubscribe"=>
        {"type"=>"boolean",
         "description"=>
          "Recipient unsubscribe from specific group, by either direct link or updating preferences. You need to enable Subscription Tracking for getting this type of event."},
       "spam_report"=>
        {"type"=>"boolean",
         "description"=>"Recipient marked a message as spam."},
       "bounce"=>
        {"type"=>"boolean",
         "description"=>
          "Receiving server could not or would not accept message."},
       "deferred"=>
        {"type"=>"boolean",
         "description"=>
          "Recipient's email server temporarily rejected message."},
       "unsubscribe"=>
        {"type"=>"boolean",
         "description"=>
          "Recipient clicked on message's subscription management link. You need to enable Subscription Tracking for getting this type of event."},
       "processed"=>
        {"type"=>"boolean",
         "description"=>
          "Message has been received and is ready to be delivered."},
       "open"=>
        {"type"=>"boolean",
         "description"=>
          "Recipient has opened the HTML message. You need to enable Open Tracking for getting this type of event."},
       "click"=>
        {"type"=>"boolean",
         "description"=>
          "Recipient clicked on a link within the message. You need to enable Click Tracking for getting this type of event."},
       "dropped"=>
        {"type"=>"boolean",
         "description"=>
          "You may see the following drop reasons: Invalid SMTPAPI header, Spam Content (if spam checker app enabled), Unsubscribed Address, Bounced Address, Spam Reporting Address, Invalid, Recipient List over Package Quota"}},
     "required"=>
      ["enabled",
       "url",
       "group_resubscribe",
       "delivered",
       "group_unsubscribe",
       "spam_report",
       "bounce",
       "deferred",
       "unsubscribe",
       "processed",
       "open",
       "click",
       "dropped"]},
   "user_profile"=>
    {"title"=>"User: Profile",
     "type"=>"object",
     "properties"=>
      {"address"=>
        {"type"=>"string",
         "description"=>"The street address for this user profile."},
       "address2"=>
        {"type"=>"string",
         "description"=>
          "An optional second line for the street address of this user profile."},
       "city"=>
        {"type"=>"string", "description"=>"The city for the user profile."},
       "company"=>
        {"type"=>"string",
         "description"=>
          "That company that this user profile is associated with."},
       "country"=>
        {"type"=>"string", "description"=>"Th country of this user profile."},
       "first_name"=>
        {"type"=>"string", "description"=>"The first name of the user."},
       "last_name"=>
        {"type"=>"string", "description"=>"The last name of the user."},
       "phone"=>
        {"type"=>"string", "description"=>"The phone number for the user."},
       "state"=>{"type"=>"string", "description"=>"The state for this user."},
       "website"=>
        {"type"=>"string",
         "description"=>"The website associated with this user."},
       "zip"=>
        {"type"=>"string", "description"=>"The zip code for this user."}},
     "example"=>
      {"address"=>"1451 Larimer Street, 3rd floor",
       "address2"=>"",
       "city"=>"Denver, CO",
       "company"=>"SendGrid",
       "country"=>"US",
       "first_name"=>"Matthew",
       "last_name"=>"Bernier",
       "phone"=>"7208788003",
       "state"=>"CO",
       "website"=>"http://sendgrid.com",
       "zip"=>"80202"}},
   "mail_settings_footer"=>
    {"title"=>"Mail Settings: Footer",
     "type"=>"object",
     "properties"=>
      {"enabled"=>
        {"type"=>"boolean",
         "description"=>
          "Indicates if the Footer mail setting is currently enabled."},
       "html_content"=>
        {"type"=>"string",
         "description"=>"The custom HTML content of your email footer."},
       "plain_content"=>
        {"type"=>"string",
         "description"=>"The plain text content of your email footer."}},
     "example"=>
      {"enabled"=>true,
       "html_content"=>"Example HTML content",
       "plain_content"=>"Example plain content"}},
   "category_stats"=>
    {"title"=>"Stats: Category Stats",
     "type"=>"object",
     "properties"=>
      {"date"=>
        {"type"=>"string",
         "description"=>"The date the statistics were gathered."},
       "stats"=>
        {"type"=>"array",
         "items"=>
          {"type"=>"object",
           "properties"=>
            {"metrics"=>
              {"type"=>"object",
               "properties"=>
                {"blocks"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of emails that were not allowed to be delivered by ISPs."},
                 "bounce_drops"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of emails that were dropped because of a bounce."},
                 "bounces"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of emails that bounced instead of being delivered."},
                 "clicks"=>
                  {"type"=>"integer",
                   "description"=>"The number of links that were clicked."},
                 "deferred"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of emails that temporarily could not be delivered."},
                 "delivered"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of emails SendGrid was able to confirm were actually delivered to a recipient."},
                 "invalid_emails"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of recipients who had malformed email addresses or whose mail provider reported the address as invalid."},
                 "opens"=>
                  {"type"=>"integer",
                   "description"=>
                    "The total number of times your emails were opened by recipients."},
                 "processed"=>
                  {"type"=>"integer",
                   "description"=>
                    "Requests from your website, application, or mail client via SMTP Relay or the API that SendGrid processed."},
                 "requests"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of emails that were requested to be delivered."},
                 "spam_report_drops"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of emails that were dropped due to a recipient previously marking your emails as spam."},
                 "spam_reports"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of recipients who marked your email as spam."},
                 "unique_clicks"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of unique recipients who clicked links in your emails."},
                 "unique_opens"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of unique recipients who opened your emails."},
                 "unsubscribe_drops"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of emails dropped due to a recipient unsubscribing from your emails."},
                 "unsubscribes"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of recipients who unsubscribed from your emails."}},
               "required"=>
                ["blocks",
                 "bounce_drops",
                 "bounces",
                 "clicks",
                 "deferred",
                 "delivered",
                 "invalid_emails",
                 "opens",
                 "processed",
                 "requests",
                 "spam_report_drops",
                 "spam_reports",
                 "unique_clicks",
                 "unique_opens",
                 "unsubscribe_drops",
                 "unsubscribes"]},
             "name"=>
              {"type"=>"string", "description"=>"The name of the category."},
             "type"=>
              {"type"=>"string",
               "description"=>"How you are segmenting your statistics."}},
           "required"=>["type"]}}},
     "required"=>["date"],
     "example"=>
      {"date"=>"2015-01-01",
       "stats"=>
        [{"metrics"=>
           {"blocks"=>0,
            "bounce_drops"=>0,
            "bounces"=>0,
            "clicks"=>0,
            "deferred"=>0,
            "delivered"=>0,
            "invalid_emails"=>0,
            "opens"=>0,
            "processed"=>0,
            "requests"=>0,
            "spam_report_drops"=>0,
            "spam_reports"=>0,
            "unique_clicks"=>0,
            "unique_opens"=>0,
            "unsubscribe_drops"=>0,
            "unsubscribes"=>0},
          "name"=>"cat1",
          "type"=>"category"},
         {"metrics"=>
           {"blocks"=>0,
            "bounce_drops"=>0,
            "bounces"=>0,
            "clicks"=>0,
            "deferred"=>0,
            "delivered"=>0,
            "invalid_emails"=>0,
            "opens"=>0,
            "processed"=>0,
            "requests"=>0,
            "spam_report_drops"=>0,
            "spam_reports"=>0,
            "unique_clicks"=>0,
            "unique_opens"=>0,
            "unsubscribe_drops"=>0,
            "unsubscribes"=>0},
          "name"=>"cat2",
          "type"=>"category"}]}},
   "transactional_template"=>
    {"title"=>"Transactional Templates: Template",
     "type"=>"object",
     "properties"=>
      {"id"=>
        {"type"=>"string",
         "description"=>"The ID of the transactional template."},
       "name"=>
        {"type"=>"string",
         "description"=>"The name for the transactional template.",
         "maxLength"=>100},
       "versions"=>
        {"type"=>"array",
         "description"=>
          "The different versions of this transactional template.",
         "items"=>{"$ref"=>"#/definitions/transactional_template_version"}}},
     "required"=>["id", "name"]},
   "parse-setting"=>
    {"title"=>"Parse Setting",
     "type"=>"object",
     "properties"=>
      {"url"=>
        {"type"=>"string",
         "description"=>
          "The public URL where you would like SendGrid to POST the data parsed from your email. Any emails sent with the given hostname provided (whose MX records have been updated to point to SendGrid) will be parsed and POSTed to this URL."},
       "hostname"=>
        {"type"=>"string",
         "description"=>
          "A specific and unique domain or subdomain that you have created to use exclusively to parse your incoming email. For example, parse.yourdomain.com."},
       "spam_check"=>
        {"type"=>"boolean",
         "description"=>
          "Indicates if you would like SendGrid to check the content parsed from your emails for spam before POSTing them to your domain."},
       "send_raw"=>
        {"type"=>"boolean",
         "description"=>
          "Indicates if you would like SendGrid to post the original MIME-type content of your parsed email. When this parameter is set to \"false\", SendGrid will send a JSON payload of the content of your email. "}},
     "example"=>
      {"url"=>"http://email.myhostname.com",
       "hostname"=>"myhostname.com",
       "spam_check"=>false,
       "send_raw"=>true}},
   "contactdb_list"=>
    {"title"=>"ContactDB lists",
     "type"=>"object",
     "properties"=>
      {"id"=>
        {"type"=>"integer", "description"=>"The reference ID of your list."},
       "name"=>{"type"=>"string", "description"=>"The name of your list."},
       "recipient_count"=>
        {"type"=>"integer",
         "description"=>"The count of recipients currently in the list."}},
     "required"=>["id", "name", "recipient_count"],
     "example"=>{"id"=>1, "name"=>"listname", "recipient_count"=>0}},
   "suppression_group"=>
    {"title"=>"Suppressions: Suppression Group",
     "type"=>"object",
     "properties"=>
      {"id"=>
        {"type"=>"number", "description"=>"The id of the suppression group."},
       "name"=>
        {"type"=>"string",
         "description"=>
          "The name of the suppression group. Each group created by a user must have a unique name.",
         "maxLength"=>30},
       "description"=>
        {"type"=>"string",
         "description"=>"A description of the suppression group.",
         "maxLength"=>100},
       "last_email_sent_at"=>{"type"=>"null"},
       "is_default"=>
        {"type"=>"boolean",
         "default"=>false,
         "description"=>
          "Indicates if this is the default suppression group."}},
     "required"=>["id", "name", "description"]},
   "mail_settings_bounce_purge"=>
    {"title"=>"Mail Settings: Bounce Purge",
     "type"=>"object",
     "properties"=>
      {"enabled"=>
        {"type"=>"boolean",
         "description"=>
          "Indicates if the bounce purge mail setting is enabled."},
       "soft_bounces"=>
        {"type"=>["integer", "null"],
         "description"=>
          "The number of days, after which SendGrid will purge all contacts from your soft bounces suppression lists."},
       "hard_bounces"=>
        {"type"=>["integer", "null"],
         "description"=>
          "The number of days, after which SendGrid will purge all contacts from your hard bounces suppression lists."}},
     "example"=>{"enabled"=>false, "soft_bounces"=>1234, "hard_bounces"=>nil}},
   "transactional_template_version"=>
    {"title"=>"Transactional Templates: Version",
     "type"=>"object",
     "properties"=>
      {"template_id"=>
        {"type"=>"string",
         "description"=>"The name of the original transactional template."},
       "active"=>
        {"type"=>"integer",
         "description"=>
          "Set the new version as the active version associated with the template. Only one version of a template can be active. The first version created for a template will automatically be set to Active.",
         "enum"=>[0, 1]},
       "name"=>
        {"type"=>"string",
         "description"=>"Name of the new transactional template version."},
       "html_content"=>
        {"type"=>"string",
         "description"=>
          "The HTML content of the new version. Must include <%body%> tag. Maximum of 1048576 bytes allowed for plain content."},
       "plain_content"=>
        {"type"=>"string",
         "description"=>
          "Text/plain content of the new transactional template version. Must include <%body%> tag. Maximum of 1048576 bytes allowed for plain content."},
       "subject"=>
        {"type"=>"string",
         "description"=>
          "Subject of the new transactional template version. Must include <%subject%> tag."}},
     "required"=>
      ["template_id",
       "active",
       "name",
       "html_content",
       "plain_content",
       "subject"]},
   "mail_settings_bcc"=>
    {"title"=>"Mail Settings: BCC",
     "type"=>"object",
     "properties"=>
      {"email"=>
        {"type"=>"string",
         "description"=>
          "The email address that will be sent a blind carbon copy."},
       "enabled"=>
        {"type"=>"boolean",
         "description"=>"Indicates if the BCC setting is enabled."}},
     "example"=>{"email"=>"example@example.com", "enabled"=>false}},
   "global:id"=>{"title"=>"Global: ID", "type"=>"integer"},
   "credentials"=>
    {"title"=>"Credentials",
     "type"=>"object",
     "properties"=>
      {"permissions"=>
        {"type"=>"object",
         "properties"=>
          {"api"=>{"type"=>"string"},
           "mail"=>{"type"=>"string"},
           "web"=>{"type"=>"string"}}},
       "username"=>{"type"=>"string"}},
     "example"=>
      {"address"=>"1234 example street",
       "address2"=>nil,
       "city"=>"Denver",
       "company"=>"Company name",
       "country"=>"US",
       "email"=>"example@example.com",
       "first_name"=>"Example",
       "last_name"=>"User",
       "phone"=>"(555) 555-5555",
       "state"=>"CO",
       "zip"=>"55555"}},
   "subuser_stats"=>
    {"title"=>"subuser_stats",
     "type"=>"object",
     "properties"=>
      {"date"=>
        {"type"=>"string",
         "description"=>"The date the statistics were gathered."},
       "stats"=>
        {"type"=>"array",
         "description"=>"The list of statistics.",
         "items"=>
          {"type"=>"object",
           "properties"=>
            {"first_name"=>
              {"type"=>"string",
               "description"=>"The first name of the subuser."},
             "last_name"=>
              {"type"=>"string",
               "description"=>"The last name of the subuser."},
             "metrics"=>
              {"type"=>"object",
               "properties"=>
                {"blocks"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of emails that were not allowed to be delivered by ISPs."},
                 "bounce_drops"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of emails that were dropped because of a bounce."},
                 "bounces"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of emails that bounced instead of being delivered."},
                 "clicks"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of links that were clicked in your emails."},
                 "deferred"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of emails that temporarily could not be delivered."},
                 "delivered"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of emails SendGrid was able to confirm were actually delivered to a recipient."},
                 "invalid_emails"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of recipients who had malformed email addresses or whose mail provider reported the address as invalid."},
                 "opens"=>
                  {"type"=>"integer",
                   "description"=>
                    "The total number of times your emails were opened by recipients."},
                 "processed"=>
                  {"type"=>"integer",
                   "description"=>
                    "Requests from your website, application, or mail client via SMTP Relay or the API that SendGrid processed."},
                 "requests"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of emails that were requested to be delivered."},
                 "spam_report_drops"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of emails that were dropped due to a recipient previously marking your emails as spam."},
                 "spam_reports"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of recipients who marked your email as spam."},
                 "unique_clicks"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of unique recipients who clicked links in your emails."},
                 "unique_opens"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of unique recipients who opened your emails."},
                 "unsubscribe_drops"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of emails dropped due to a recipient unsubscribing from your emails."},
                 "unsubscribes"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of recipients who unsubscribed from your emails."}}},
             "name"=>
              {"type"=>"string",
               "description"=>"The username of the subuser."},
             "type"=>
              {"type"=>"string", "description"=>"The type of account."}}}}}},
   "campaign_request"=>
    {"title"=>"Campaigns Request",
     "type"=>"object",
     "properties"=>
      {"title"=>
        {"type"=>"string",
         "description"=>
          "The display title of your campaign. This will be viewable by you in the Marketing Campaigns UI."},
       "subject"=>
        {"type"=>["string", "null"],
         "description"=>
          "The subject of your campaign that your recipients will see."},
       "sender_id"=>
        {"type"=>["null", "integer"],
         "description"=>
          "The ID of the \"sender\" identity that you have created. Your recipients will see this as the \"from\" on your marketing emails."},
       "list_ids"=>
        {"type"=>["array", "null"],
         "description"=>
          "The IDs of the lists you are sending this campaign to. You can have both segment IDs and list IDs",
         "items"=>{"type"=>"integer"}},
       "segment_ids"=>
        {"type"=>["array", "null"],
         "description"=>
          "The segment IDs that you are sending this list to. You can have both segment IDs and list IDs.",
         "items"=>{"type"=>"integer"}},
       "categories"=>
        {"type"=>["array", "null"],
         "description"=>
          "The categories you would like associated to this campaign.",
         "items"=>{"type"=>"string"}},
       "suppression_group_id"=>
        {"type"=>["null", "integer"],
         "description"=>
          "The suppression group that this marketing email belongs to, allowing recipients to opt-out of emails of this type."},
       "custom_unsubscribe_url"=>
        {"type"=>["string", "null"],
         "description"=>
          "This is the url of the custom unsubscribe page that you provide for customers to unsubscribe from your suppression groups."},
       "ip_pool"=>
        {"type"=>["string", "null"],
         "description"=>
          "The pool of IPs that you would like to send this email from."},
       "html_content"=>
        {"type"=>["string", "null"],
         "description"=>"The HTML of your marketing email."},
       "plain_content"=>
        {"type"=>["string", "null"],
         "description"=>"The plain text content of your emails."}},
     "required"=>["title"],
     "example"=>
      {"id"=>986724,
       "title"=>"May Newsletter",
       "subject"=>"New Products for Summer!",
       "sender_id"=>124451,
       "list_ids"=>[110, 124],
       "segment_ids"=>[110],
       "categories"=>["summer line"],
       "suppression_group_id"=>42,
       "custom_unsubscribe_url"=>"",
       "ip_pool"=>"marketing",
       "html_content"=>
        "<html><head><title></title></head><body><p>Check out our summer line!</p></body></html>",
       "plain_content"=>"Check out our summer line!",
       "status"=>"Draft"}},
   "user_scheduled_send_status"=>
    {"title"=>"User: Scheduled Send status",
     "allOf"=>
      [{"$ref"=>"#/definitions/mail_batch_id"},
       {"type"=>"object",
        "description"=>"The status of the scheduled send.",
        "properties"=>
         {"status"=>
           {"type"=>"string",
            "description"=>"The status of the scheduled send.",
            "enum"=>["cancel", "pause"]}},
        "required"=>["status"]}]},
   "mail_settings_forward_spam"=>
    {"title"=>"Mail Settings: Forward Spam",
     "type"=>"object",
     "properties"=>
      {"email"=>
        {"type"=>"string",
         "description"=>
          "The email address where you would like the spam reports to be forwarded."},
       "enabled"=>
        {"type"=>"boolean",
         "description"=>"Indicates if the Forward Spam setting is enabled."}},
     "example"=>{"email"=>"", "enabled"=>true}},
   "contactdb_segments_with_id"=>
    {"title"=>"ContactDB:: Segments with ID",
     "allOf"=>
      [{"type"=>"object",
        "properties"=>
         {"id"=>{"type"=>"number", "description"=>"The ID of the segment."}},
        "required"=>["id"]},
       {"$ref"=>"#/definitions/contactdb_segments"}]},
   "advanced_stats_country"=>
    {"title"=>"Stats: Advanced Stats with Clicks and Opens",
     "type"=>"object",
     "properties"=>
      {"date"=>
        {"type"=>"string",
         "description"=>"The date that the events occurred."},
       "stats"=>
        {"type"=>"array",
         "description"=>"The statistics of the email events.",
         "items"=>
          {"type"=>"object",
           "properties"=>
            {"type"=>
              {"type"=>"string", "description"=>"The type of segmentation."},
             "name"=>
              {"type"=>"string",
               "description"=>"The name of the specific segmentation."},
             "metrics"=>
              {"type"=>"object",
               "description"=>"The individual events and their stats.",
               "required"=>
                ["clicks", "opens", "unique_clicks", "unique_opens"],
               "properties"=>
                {"clicks"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of links that were clicked in your emails."},
                 "opens"=>
                  {"type"=>"integer",
                   "description"=>
                    "The total number of times your emails were opened by recipients."},
                 "unique_clicks"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of unique recipients who clicked links in your emails."},
                 "unique_opens"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of unique recipients who opened your emails."}}}},
           "required"=>["type", "name", "metrics"]}}},
     "required"=>["date", "stats"]},
   "advanced_stats_clicks"=>
    {"title"=>"Stats: Advanced Stats with Clicks",
     "type"=>"object",
     "properties"=>
      {"date"=>
        {"type"=>"string",
         "description"=>"The date that the events occurred."},
       "stats"=>
        {"type"=>"array",
         "description"=>"The statistics of the email events.",
         "items"=>
          {"type"=>"object",
           "properties"=>
            {"type"=>
              {"type"=>"string", "description"=>"The type of segmentation."},
             "name"=>
              {"type"=>"string",
               "description"=>"The name of the specific segmentation."},
             "metrics"=>
              {"type"=>"object",
               "description"=>"The individual events and their stats.",
               "required"=>["clicks", "unique_clicks"],
               "properties"=>
                {"clicks"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of links that were clicked in your emails."},
                 "unique_clicks"=>
                  {"type"=>"integer",
                   "description"=>
                    "The number of unique recipients who clicked links in your emails."}}}},
           "required"=>["type", "name", "metrics"]}}},
     "required"=>["date", "stats"]},
   "contactdb_recipient"=>
    {"title"=>"ContactDB: Recipient",
     "type"=>"object",
     "properties"=>
      {"recipients"=>
        {"type"=>"array",
         "items"=>
          {"type"=>"object",
           "properties"=>
            {"id"=>
              {"type"=>"string", "description"=>"The ID of this recipient."},
             "created_at"=>
              {"type"=>"number",
               "description"=>
                "The time this record was created in your contactdb, in unixtime."},
             "custom_fields"=>
              {"type"=>"array",
               "description"=>
                "The custom fields assigned to this recipient and their values.",
               "items"=>
                {"$ref"=>
                  "#/definitions/contactdb_custom_field_with_id_value"}},
             "email"=>
              {"type"=>"string",
               "description"=>
                "The email address of this recipient. This is a default custom field that SendGrid provides.",
               "format"=>"email"},
             "first_name"=>
              {"type"=>["string", "null"],
               "description"=>
                "The first name of this recipient. This is a default custom field that SendGrid provides."},
             "last_name"=>
              {"type"=>["string", "null"],
               "description"=>"The last name of the recipient."},
             "last_clicked"=>
              {"type"=>["number", "null"],
               "description"=>
                "The last time this recipient clicked a link from one of your campaigns, in unixtime."},
             "last_emailed"=>
              {"type"=>["number", "null"],
               "description"=>
                "The last time this user was emailed by one of your campaigns, in unixtime."},
             "last_opened"=>
              {"type"=>["number", "null"],
               "description"=>
                "The last time this recipient opened an email from you, in unixtime."},
             "updated_at"=>
              {"type"=>"number",
               "description"=>
                "The last update date for this recipient's record."}},
           "required"=>["email"]}}}},
   "mail_settings_patch"=>
    {"title"=>"Mail Settings: Patch",
     "type"=>"object",
     "properties"=>
      {"enabled"=>
        {"type"=>"boolean",
         "description"=>"Indicates if the mail setting is enabled."},
       "email"=>
        {"type"=>"string",
         "description"=>"The email address of the recipient."}},
     "example"=>{"enabled"=>true, "email"=>"email@example.com"}},
   "mail_settings_forward_bounce"=>
    {"title"=>"Mail Settings: Forward Bounce",
     "type"=>"object",
     "properties"=>
      {"email"=>
        {"type"=>["string", "null"],
         "description"=>
          "The email address that you would like your bounce reports forwarded to."},
       "enabled"=>
        {"type"=>"boolean",
         "description"=>
          "Indicates if the bounce forwarding mail setting is enabled."}},
     "example"=>{"enabled"=>false, "email"=>nil}},
   "contactdb_custom_field_with_id_value"=>
    {"title"=>"ContactDB Custom field schema.",
     "allOf"=>
      [{"$ref"=>"#/definitions/contactdb_custom_field_with_id"},
       {"type"=>"object",
        "properties"=>
         {"value"=>
           {"type"=>["string", "null"],
            "description"=>"The value of this recipient's custom field"}}}]},
   "contactdb_recipient_count"=>
    {"title"=>"ContactDB: Recipient Count",
     "type"=>"object",
     "properties"=>
      {"recipient_count"=>
        {"type"=>"number", "description"=>"The count of recipients."}},
     "required"=>["recipient_count"],
     "example"=>{"recipient_count"=>1234}},
   "subuser_post"=>
    {"title"=>"Subuser::POST",
     "type"=>"object",
     "properties"=>
      {"username"=>
        {"type"=>"string", "description"=>"The username of the subuser."},
       "user_id"=>
        {"type"=>"number", "description"=>"The user ID for this subuser."},
       "email"=>
        {"type"=>"string",
         "description"=>"The email address for this subuser.",
         "format"=>"email"},
       "signup_session_token"=>{"type"=>"string"},
       "authorization_token"=>{"type"=>"string"},
       "credit_allocation"=>
        {"type"=>"object", "properties"=>{"type"=>{"type"=>"string"}}}},
     "required"=>["username", "user_id", "email"],
     "example"=>
      {"username"=>"example_subuser",
       "user_id"=>1234,
       "email"=>"example@example.com",
       "signup_session_token"=>"",
       "authorization_token"=>"",
       "credit_allocation"=>{"type"=>"unlimited"}}},
   "whitelabel::domain"=>
    {"title"=>"Whitelabel - Domain",
     "type"=>"object",
     "properties"=>
      {"id"=>
        {"type"=>"number", "description"=>"The ID of the domain whitelabel."},
       "user_id"=>
        {"type"=>"number",
         "description"=>
          "The ID of the user that this whitelabel will be associated with."},
       "subdomain"=>
        {"type"=>"string",
         "description"=>"The subdomain to use for this domain whitelabel."},
       "domain"=>
        {"type"=>"string",
         "description"=>
          "The domain that this whitelabel is being created for."},
       "username"=>
        {"type"=>"string",
         "description"=>
          "The username that this whitelabel will be associated with."},
       "ips"=>
        {"type"=>"array",
         "description"=>
          "The IPs to be included in the custom SPF record for this domain whitelabel.",
         "items"=>{"type"=>"object"}},
       "custom_spf"=>
        {"type"=>"boolean",
         "description"=>
          "Indicates whether this domain whitelabel will use custom SPF."},
       "default"=>
        {"type"=>"boolean",
         "description"=>
          "Indicates if this domain whitelabel is the default whitelabel."},
       "legacy"=>
        {"type"=>"boolean",
         "description"=>
          "Indicates if this domain whitelabel was created using the legacy whitelabel tool."},
       "automatic_security"=>
        {"type"=>"boolean",
         "description"=>
          "Indicates if this domain whitelabel uses automated security."},
       "valid"=>
        {"type"=>"boolean",
         "description"=>"Indicates if this is a valid whitelabel."},
       "dns"=>
        {"type"=>"object",
         "description"=>
          "The DNS records for this whitelabel that are used to authenticate the sending domain.",
         "required"=>["mail_cname", "dkim1", "dkim2"],
         "properties"=>
          {"mail_cname"=>
            {"type"=>"object",
             "description"=>
              "The CNAME for your sending domain that points to sendgrid.net.",
             "required"=>["valid", "type", "host", "data"],
             "properties"=>
              {"valid"=>
                {"type"=>"boolean",
                 "description"=>"Indicates if this is a valid CNAME."},
               "type"=>
                {"type"=>"string", "description"=>"The type of DNS record."},
               "host"=>
                {"type"=>"string",
                 "description"=>"The domain that this CNAME is created for.",
                 "format"=>"hostname"},
               "data"=>
                {"type"=>"string", "description"=>"The CNAME record."}}},
           "dkim1"=>
            {"type"=>"object",
             "description"=>"A DNS record.",
             "required"=>["valid", "type", "host", "data"],
             "properties"=>
              {"valid"=>
                {"type"=>"boolean",
                 "description"=>"Indicates if this is a valid DNS record."},
               "type"=>
                {"type"=>"string", "description"=>"The type of DNS record."},
               "host"=>
                {"type"=>"string",
                 "description"=>
                  "The domain that this DNS record was created for."},
               "data"=>{"type"=>"string", "description"=>"The DNS record."}}},
           "dkim2"=>
            {"type"=>"object",
             "description"=>"A DNS record.",
             "required"=>["valid", "type", "host", "data"],
             "properties"=>
              {"valid"=>
                {"type"=>"boolean",
                 "description"=>"Indicates if this is a valid DNS record."},
               "type"=>
                {"type"=>"string", "description"=>"The type of DNS record."},
               "host"=>
                {"type"=>"string",
                 "description"=>
                  "The domain that this DNS record was created for."},
               "data"=>
                {"type"=>"string", "description"=>"The DNS record."}}}}}},
     "required"=>
      ["id",
       "user_id",
       "subdomain",
       "domain",
       "username",
       "ips",
       "custom_spf",
       "default",
       "legacy",
       "automatic_security",
       "valid",
       "dns"]},
   "mail_batch_id"=>
    {"title"=>"Mail: Batch ID",
     "type"=>"object",
     "properties"=>
      {"batch_id"=>{"type"=>"string", "pattern"=>"^[a-zA-Z0-9\\-\\_]"}},
     "required"=>["batch_id"],
     "example"=>{"batch_id"=>"HkJ5yLYULb7Rj8GKSx7u025ouWVlMgAi"}}}}
end
